/*
 * Copyright (C) 2012 by
 *   MetraLabs GmbH (MLAB), GERMANY
 * and
 *   Neuroinformatics and Cognitive Robotics Labs (NICR) at TU Ilmenau, GERMANY
 * All rights reserved.
 *
 * Contact: info@mira-project.org
 *
 * Commercial Usage:
 *   Licensees holding valid commercial licenses may use this file in
 *   accordance with the commercial license agreement provided with the
 *   software or, alternatively, in accordance with the terms contained in
 *   a written agreement between you and MLAB or NICR.
 *
 * GNU General Public License Usage:
 *   Alternatively, this file may be used under the terms of the GNU
 *   General Public License version 3.0 as published by the Free Software
 *   Foundation and appearing in the file LICENSE.GPL3 included in the
 *   packaging of this file. Please review the following information to
 *   ensure the GNU General Public License version 3.0 requirements will be
 *   met: http://www.gnu.org/copyleft/gpl.html.
 *   Alternatively you may (at your option) use any later version of the GNU
 *   General Public License if such license has been publicly approved by
 *   MLAB and NICR (or its successors, if any).
 *
 * IN NO EVENT SHALL "MLAB" OR "NICR" BE LIABLE TO ANY PARTY FOR DIRECT,
 * INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
 * THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF "MLAB" OR
 * "NICR" HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * "MLAB" AND "NICR" SPECIFICALLY DISCLAIM ANY WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE PROVIDED HEREUNDER IS
 * ON AN "AS IS" BASIS, AND "MLAB" AND "NICR" HAVE NO OBLIGATION TO
 * PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS OR MODIFICATIONS.
 */

#include <boost/test/unit_test.hpp>

#include <iostream>
#include <thread/ScopedAccess.h>

#include <utility>

using namespace mira;

///////////////////////////////////////////////////////////////////////////////

class MyObject
{
public:

	MyObject() {}

	void lock() {
		std::cout << "locked " << this << std::endl;
		mMutex.lock();
	}

	void unlock() {
		std::cout << "unlocked " << this << std::endl;
		mMutex.unlock();
	}

	typedef ScopedAccess<MyObject> Access;
	typedef ScopedAccess<MyObject, boost::mutex> Access2;

	Access getAccess() { return Access(this); }
	Access2 getAccess2() { return Access2(this, &mMutex); }

	void foo() {
		std::cout << "foo: " << this << std::endl;
	}

	bool isLocked() {
		if (!mMutex.try_lock())
			return true;
		mMutex.unlock();
		return false;
	}

public:
	boost::mutex mMutex;
};

///////////////////////////////////////////////////////////////////////////////

BOOST_AUTO_TEST_CASE( InterfaceTest )
{
	MyObject obj;

	BOOST_CHECK(!obj.isLocked());
	{
		std::cout << "entered scope" << std::endl;
		MyObject::Access a = obj.getAccess();

		BOOST_CHECK(obj.isLocked());

		a->foo();
		MyObject& obj = *a;

		std::cout << "leaving scope" << std::endl;
		BOOST_CHECK(obj.isLocked());
	}

	BOOST_CHECK(!obj.isLocked());

	{
		std::cout << "entered scope" << std::endl;
		MyObject::Access2 a = obj.getAccess2();

		BOOST_CHECK(obj.isLocked());

		a->foo();
		MyObject& obj = *a;

		std::cout << "leaving scope" << std::endl;
		BOOST_CHECK(obj.isLocked());
	}
}
